@@ -12,6 +12,9 @@
 **    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
 **    David Barth <david.barth@canonical.com>
 **
+** Contributor(s):
+**    Chow Loong Jin <hyperair@gmail.com> (fix for LP: #401809, rev. 349)
+**
 ** This program is free software: you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License version 3, as published
 ** by the Free Software Foundation.
@@ -34,9 +37,11 @@
 #include <gconf/gconf-client.h>
 #include <libwnck/application.h>
 #include <libwnck/class-group.h>
+#include <libwnck/window.h>
 #include <libwnck/workspace.h>
 
 #include "defaults.h"
+#include "util.h"
 
 G_DEFINE_TYPE (Defaults, defaults, G_TYPE_OBJECT);
 
@@ -66,6 +71,8 @@ enum
 	PROP_CONTENT_SHADOW_COLOR,
 	PROP_MARGIN_SIZE,
 	PROP_ICON_SIZE,
+	PROP_GAUGE_SIZE,
+	PROP_GAUGE_OUTLINE_WIDTH,
 	PROP_FADE_IN_TIMEOUT,
 	PROP_FADE_OUT_TIMEOUT,
 	PROP_ON_SCREEN_TIMEOUT,
@@ -76,12 +83,16 @@ enum
 	PROP_TEXT_BODY_COLOR,
 	PROP_TEXT_BODY_WEIGHT,
 	PROP_TEXT_BODY_SIZE,
-	PROP_PIXELS_PER_EM
+	PROP_PIXELS_PER_EM,
+	PROP_SYSTEM_FONT_SIZE,
+	PROP_SCREEN_DPI,
+	PROP_GRAVITY
 };
 
 enum
 {
 	VALUE_CHANGED,
+	GRAVITY_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -103,7 +114,7 @@ enum
 /* these values are interpreted as em-measurements and do comply to the 
  * visual guide for jaunty-notifications */
 #define DEFAULT_DESKTOP_BOTTOM_GAP    6.0f
-#define DEFAULT_BUBBLE_WIDTH         23.0f
+#define DEFAULT_BUBBLE_WIDTH         24.0f
 #define DEFAULT_BUBBLE_MIN_HEIGHT     5.0f
 #define DEFAULT_BUBBLE_MAX_HEIGHT    12.2f
 #define DEFAULT_BUBBLE_VERT_GAP       0.5f
@@ -116,22 +127,27 @@ enum
 #define DEFAULT_BUBBLE_CORNER_RADIUS 0.375f
 #define DEFAULT_CONTENT_SHADOW_SIZE  0.125f
 #define DEFAULT_CONTENT_SHADOW_COLOR "#000000"
-#define DEFAULT_MARGIN_SIZE          1.4f
-#define DEFAULT_ICON_SIZE            4.0f
+#define DEFAULT_MARGIN_SIZE          1.0f
+#define DEFAULT_ICON_SIZE            3.0f
+#define DEFAULT_GAUGE_SIZE           0.625f
+#define DEFAULT_GAUGE_OUTLINE_WIDTH  0.125f
 #define DEFAULT_TEXT_FONT_FACE       "Sans"
 #define DEFAULT_TEXT_TITLE_COLOR     "#ffffff"
 #define DEFAULT_TEXT_TITLE_WEIGHT    TEXT_WEIGHT_BOLD
-#define DEFAULT_TEXT_TITLE_SIZE      0.95f
+#define DEFAULT_TEXT_TITLE_SIZE      1.0f
 #define DEFAULT_TEXT_BODY_COLOR      "#eaeaea"
 #define DEFAULT_TEXT_BODY_WEIGHT     TEXT_WEIGHT_NORMAL
-#define DEFAULT_TEXT_BODY_SIZE       0.8f
-#define DEFAULT_PIXELS_PER_EM        10
+#define DEFAULT_TEXT_BODY_SIZE       0.9f
+#define DEFAULT_PIXELS_PER_EM        10.0f
+#define DEFAULT_SYSTEM_FONT_SIZE     10.0f
+#define DEFAULT_SCREEN_DPI           96.0f
+#define DEFAULT_GRAVITY              GRAVITY_NORTH_EAST
 
 /* these values are interpreted as milliseconds-measurements and do comply to
  * the visual guide for jaunty-notifications */
 #define DEFAULT_FADE_IN_TIMEOUT      250
 #define DEFAULT_FADE_OUT_TIMEOUT     1000
-#define DEFAULT_ON_SCREEN_TIMEOUT    5000
+#define DEFAULT_ON_SCREEN_TIMEOUT    10000
 
 /* GConf-keys to watch */
 #define GCONF_UI_FONT_NAME        "/desktop/gnome/interface/font_name"
@@ -143,9 +159,11 @@ enum
 /* GConf-trees to watch */
 #define GCONF_UI_TREE             "/desktop/gnome/interface"
 #define GCONF_FONT_TREE           "/desktop/gnome/font_rendering"
+#define GCONF_NOSD_TREE           "/apps/notify-osd"
 
 /* notify-osd settings */
 #define GCONF_MULTIHEAD_MODE "/apps/notify-osd/multihead_mode"
+#define GCONF_GRAVITY        "/apps/notify-osd/gravity"
 
 static guint g_defaults_signals[LAST_SIGNAL] = { 0 };
 
@@ -154,64 +172,40 @@ static guint g_defaults_signals[LAST_SIGNAL] = { 0 };
 static void
 _get_font_size_dpi (Defaults* self)
 {
-	GString*   string        = NULL;
-	GError*    error         = NULL;
-	GScanner*  scanner       = NULL;
-	GTokenType token         = G_TOKEN_NONE;
-	gint       points        = 0;
-	GString*   font_face     = NULL;
-	gdouble    dpi           = 0.0f;
-	gint       pixels_per_em = 0;
+	GString*    string        = NULL;
+	GError*     error         = NULL;
+	guint       points        = 0;
+	GString*    font_face     = NULL;
+	gdouble     dpi           = 0.0f;
+	gdouble     pixels_per_em = 0;
+	gchar*      font_name     = NULL;
 
 	if (!IS_DEFAULTS (self))
 		return;
 
 	/* determine current system font-name/size */
 	error = NULL;
-	string = g_string_new (gconf_client_get_string (self->context,
-							GCONF_UI_FONT_NAME,
-							&error));
+	font_name = gconf_client_get_string (self->context,
+					     GCONF_UI_FONT_NAME,
+					     &error);
+	string = g_string_new (font_name);
 	if (error)
 	{
-		/* if something went wrong, assume "Sans 10" and continue */
+		// if something went wrong, assume "Sans 10" and continue
 		string = g_string_assign (string, "Sans 10");
+
+		g_warning ("_get_font_size_dpi(): Got error \"%s\"\n",
+		           error->message);
+		g_error_free (error);
 	}
+	g_free ((gpointer) font_name);
 
-	/* extract font-family-name and font-size */
-	scanner = g_scanner_new (NULL);
-	if (scanner)
-	{
-		g_scanner_input_text (scanner, string->str, string->len);
-		for (token = g_scanner_get_next_token (scanner);
-		     token != G_TOKEN_EOF;
-		     token = g_scanner_get_next_token (scanner))
-		{
-			switch (token)
-			{
-				case G_TOKEN_INT:
-					points = (gint) scanner->value.v_int;
-				break;
+	// extract text point-size
+	points = extract_point_size (string->str);
 
-				case G_TOKEN_IDENTIFIER:
-					if (!font_face)
-						font_face = g_string_new (scanner->value.v_string);
-					else
-					{
-						g_string_append (font_face,
-								 " ");
-						g_string_append (font_face,
-								 scanner->value.v_string);
-					}
-				break;
+	// extract font-face-name/style
+	font_face = extract_font_face (string->str);
 
-				default:
-				break;
-			}
-		}
-		g_scanner_destroy (scanner);
-	}
-
-	/* clean up */
 	if (string != NULL)
 		g_string_free (string, TRUE);
 
@@ -222,18 +216,68 @@ _get_font_size_dpi (Defaults* self)
 		g_string_free (font_face, TRUE);
 	}
 
+	/* update stored system-font size (in pt!) */
+	g_object_set (self, "system-font-size", (gdouble) points, NULL);
+
 	/* determine current system DPI-setting */
 	error = NULL;
 	dpi = gconf_client_get_float (self->context, GCONF_FONT_DPI, &error);
 	if (error)
 	{
-		/* if something went wrong, assume 72 DPI and continue */
-		dpi = 72.0f;
+		// if something went wrong, assume 96 DPI and continue
+		dpi = DEFAULT_SCREEN_DPI;
+
+		g_warning ("_get_font_size_dpi(): Got error \"%s\"\n",
+		           error->message);
+		g_error_free (error);
 	}
 
 	/* update stored DPI-value */
-	pixels_per_em = (gint) ((gdouble) points * 1.0f / 72.0f * dpi);
-	g_object_set (self, "pixels-per-em", (gint) pixels_per_em, NULL);
+	pixels_per_em = (gdouble) points * dpi / 72.0f;
+	g_object_set (self, "pixels-per-em", pixels_per_em, NULL);
+	g_object_set (self, "screen-dpi", dpi, NULL);
+
+	if (g_getenv ("DEBUG"))
+		g_print ("font-size: %dpt\ndpi: %3.1f\npixels/EM: %2.2f\nwidth: %d px\ntitle-height: %2.2f pt\nbody-height: %2.2f pt\n\n",
+			 points,
+			 defaults_get_screen_dpi (self),
+			 pixels_per_em,
+			 (gint) (pixels_per_em * DEFAULT_BUBBLE_WIDTH),
+			 defaults_get_system_font_size (self) *
+			 defaults_get_text_title_size (self),
+			 defaults_get_system_font_size (self) *
+			 defaults_get_text_body_size (self));
+}
+
+static void
+_get_gravity (Defaults* self)
+{
+	GError* error = NULL;
+	Gravity gravity;
+
+	if (!IS_DEFAULTS (self))
+		return;
+
+	// grab current gravity-setting for notify-osd from gconf
+	error = NULL;
+	gravity = gconf_client_get_int (self->context, GCONF_GRAVITY, &error);
+	if (error)
+	{
+		// make sure we use a sane default for the gravity
+		gravity = DEFAULT_GRAVITY;
+
+		g_warning ("%s(): Got error \"%s\"\n",
+		           G_STRFUNC,
+		           error->message);
+		g_error_free (error);
+	}
+
+	// protect against out-of-bounds values for gravity
+	if (gravity != GRAVITY_EAST && gravity != GRAVITY_NORTH_EAST)
+		gravity = DEFAULT_GRAVITY;
+
+	// update stored DPI-value
+	g_object_set (self, "gravity", gravity, NULL);
 }
 
 static void
@@ -338,61 +382,25 @@ _subpixel_order_changed (GConfClient* client,
 	g_signal_emit (defaults, g_defaults_signals[VALUE_CHANGED], 0);
 }
 
-static gdouble
-_get_average_char_width (Defaults* self)
+static void
+_gravity_changed (GConfClient* client,
+                  guint        cnxn_id,
+                  GConfEntry*  entry,
+                  gpointer     data)
 {
-	cairo_surface_t*      surface;
-	cairo_t*              cr;
-	PangoFontDescription* desc;
-	PangoLayout*          layout;
-	PangoContext*         context;
-	PangoLanguage*        language;
-	PangoFontMetrics*     metrics;
-	gint                  char_width;
+	Defaults* defaults;
 
-	if (!self || !IS_DEFAULTS (self))
-		return 0;
+	if (!data)
+		return;
 
-	surface = cairo_image_surface_create (CAIRO_FORMAT_A1, 1, 1);
-	if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS)
-		return 0;
+	defaults = (Defaults*) data;
+	if (!IS_DEFAULTS (defaults))
+		return;
 
-	cr = cairo_create (surface);
-	cairo_surface_destroy (surface);
-	if (cairo_status (cr) != CAIRO_STATUS_SUCCESS)
-		return 0;
+    	// grab gravity setting for notify-osd from gconf
+	_get_gravity (defaults);
 
-	layout = pango_cairo_create_layout (cr);
-	desc = pango_font_description_new ();
-	pango_font_description_set_size (
-		desc,
-		EM2PIXELS (defaults_get_text_title_size (self), self) *
-		PANGO_SCALE);
-
-	pango_font_description_set_family_static (
-		desc,
-		defaults_get_text_font_face (self));
-
-	pango_font_description_set_weight (
-		desc,
-		defaults_get_text_title_weight (self));
-
-	pango_font_description_set_style (desc, PANGO_STYLE_NORMAL);
-	pango_layout_set_wrap (layout, PANGO_WRAP_WORD);
-	pango_layout_set_font_description (layout, desc);
-
-	context  = pango_layout_get_context (layout); /* no need to unref */
-	language = pango_language_get_default ();     /* no need to unref */
-	metrics  = pango_context_get_metrics (context, desc, language);
-	char_width = pango_font_metrics_get_approximate_char_width (metrics);
-
-	/* clean up */
-	pango_font_metrics_unref (metrics);
-	pango_font_description_free (desc);
-	g_object_unref (layout);
-	cairo_destroy (cr);
-
-	return PIXELS2EM (char_width / PANGO_SCALE, self);
+	g_signal_emit (defaults, g_defaults_signals[GRAVITY_CHANGED], 0);
 }
 
 void
@@ -470,9 +478,6 @@ defaults_constructed (GObject* gobject)
 	gdouble      icon_size;
 	gdouble      bubble_height;
 	gdouble      new_bubble_height;
-	gdouble      bubble_width;
-	gdouble      new_bubble_width;
-	gdouble      average_char_width;
 
 	self = DEFAULTS (gobject);
 
@@ -480,6 +485,7 @@ defaults_constructed (GObject* gobject)
 
 	/* grab system-wide font-face/size and DPI */
 	_get_font_size_dpi (self);
+	_get_gravity (self);
 
 	/* correct the default min. bubble-height, according to the icon-size */
 	g_object_get (self,
@@ -513,31 +519,6 @@ defaults_constructed (GObject* gobject)
 			      NULL);
 	}
 
-	/* correct the default bubble-width depending on the average width of a 
-	 * character rendered in the default system-font at the default
-	 * font-size,
-	 * as default layout, we'll take the icon+title+body+message case, thus
-	 * seen from left to right we use:
-	 *
-	 *      margin + icon_size + margin + 20 * avg_char_width + margin
-	 */
-	g_object_get (self,
-		      "bubble-width",
-		      &bubble_width,
-		      NULL);
-	average_char_width = _get_average_char_width (self);
-
-	new_bubble_width = 3.0f * margin_size +
-			   icon_size +
-			   20.0f * average_char_width;
-	if (new_bubble_width > bubble_width)
-	{
-		g_object_set (self,
-			      "bubble-width",
-			      new_bubble_width,
-			      NULL);
-	}
-
 	/* FIXME: calling this here causes a segfault */
 	/* chain up to the parent class */
 	/*G_OBJECT_CLASS (defaults_parent_class)->constructed (gobject);*/
@@ -555,21 +536,91 @@ defaults_dispose (GObject* gobject)
 	gconf_client_notify_remove (defaults->context, defaults->notifier[2]);
 	gconf_client_notify_remove (defaults->context, defaults->notifier[3]);
 	gconf_client_notify_remove (defaults->context, defaults->notifier[4]);
+	gconf_client_notify_remove (defaults->context, defaults->notifier[5]);
 	gconf_client_remove_dir (defaults->context, GCONF_UI_TREE, NULL);
 	gconf_client_remove_dir (defaults->context, GCONF_FONT_TREE, NULL);
+	gconf_client_remove_dir (defaults->context, GCONF_NOSD_TREE, NULL);
 	g_object_unref (defaults->context);
 
-	/* chain up to the parent class */
+	if (defaults->bubble_shadow_color)
+	{
+		g_string_free (defaults->bubble_shadow_color, TRUE);
+		defaults->bubble_shadow_color = NULL;
+	}
+
+	if (defaults->bubble_bg_color)
+	{
+		g_string_free (defaults->bubble_bg_color, TRUE);
+		defaults->bubble_bg_color = NULL;
+	}
+
+	if (defaults->bubble_bg_opacity)
+	{
+		g_string_free (defaults->bubble_bg_opacity, TRUE);
+		defaults->bubble_bg_opacity = NULL;
+	}
+
+	if (defaults->bubble_hover_opacity)
+	{
+		g_string_free (defaults->bubble_hover_opacity, TRUE);
+		defaults->bubble_hover_opacity = NULL;
+	}
+
+	if (defaults->content_shadow_color)
+	{
+		g_string_free (defaults->content_shadow_color, TRUE);
+		defaults->content_shadow_color = NULL;
+	}
+
+	if (defaults->text_font_face)
+	{
+		g_string_free (defaults->text_font_face, TRUE);
+		defaults->text_font_face = NULL;
+	}
+
+	if (defaults->text_title_color)
+	{
+		g_string_free (defaults->text_title_color, TRUE);
+		defaults->text_title_color = NULL;
+	}
+
+	if (defaults->text_body_color)
+	{
+		g_string_free (defaults->text_body_color, TRUE);
+		defaults->text_body_color = NULL;
+	}
+
+	// chain up to the parent class
 	G_OBJECT_CLASS (defaults_parent_class)->dispose (gobject);
 }
 
 static void
 defaults_finalize (GObject* gobject)
 {
-	/* chain up to the parent class */
+	// chain up to the parent class
 	G_OBJECT_CLASS (defaults_parent_class)->finalize (gobject);
 }
 
+static void
+_handle_error (Defaults* self,
+               GError*   error)
+{
+	// no sanity-checks here
+
+	gconf_client_notify_remove (self->context, self->notifier[0]);
+	gconf_client_notify_remove (self->context, self->notifier[1]);
+	gconf_client_notify_remove (self->context, self->notifier[2]);
+	gconf_client_notify_remove (self->context, self->notifier[3]);
+	gconf_client_notify_remove (self->context, self->notifier[4]);
+	gconf_client_notify_remove (self->context, self->notifier[5]);
+	gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
+	gconf_client_remove_dir (self->context, GCONF_FONT_TREE, NULL);
+	gconf_client_remove_dir (self->context, GCONF_NOSD_TREE, NULL);
+	g_object_unref (self->context);
+	g_warning ("%s(): Got error \"%s\"\n", G_STRFUNC, error->message);
+	g_error_free (error);
+}
+
 static void
 defaults_init (Defaults* self)
 {
@@ -591,8 +642,7 @@ defaults_init (Defaults* self)
 			      &error);
 	if (error)
 	{
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
 		return;
 	}
 
@@ -604,9 +654,19 @@ defaults_init (Defaults* self)
 			      &error);
 	if (error)
 	{
-		gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
+		return;
+	}
+
+	/* register watching all settings for notify-osd */
+	error = NULL;
+	gconf_client_add_dir (self->context,
+			      GCONF_NOSD_TREE,
+			      GCONF_CLIENT_PRELOAD_NONE,
+			      &error);
+	if (error)
+	{
+		_handle_error (self, error);
 		return;
 	}
 
@@ -620,10 +680,7 @@ defaults_init (Defaults* self)
 						     &error);
 	if (error)
 	{
-		gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
-		gconf_client_remove_dir (self->context, GCONF_FONT_TREE, NULL);
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
 		return;
 	}
 
@@ -637,11 +694,7 @@ defaults_init (Defaults* self)
 						     &error);
 	if (error)
 	{
-		gconf_client_notify_remove (self->context, self->notifier[0]);
-		gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
-		gconf_client_remove_dir (self->context, GCONF_FONT_TREE, NULL);
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
 		return;
 	}
 
@@ -655,12 +708,7 @@ defaults_init (Defaults* self)
 						     &error);
 	if (error)
 	{
-		gconf_client_notify_remove (self->context, self->notifier[0]);
-		gconf_client_notify_remove (self->context, self->notifier[1]);
-		gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
-		gconf_client_remove_dir (self->context, GCONF_FONT_TREE, NULL);
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
 		return;
 	}
 	
@@ -674,13 +722,7 @@ defaults_init (Defaults* self)
 						     &error);
 	if (error)
 	{
-		gconf_client_notify_remove (self->context, self->notifier[0]);
-		gconf_client_notify_remove (self->context, self->notifier[1]);
-		gconf_client_notify_remove (self->context, self->notifier[2]);
-		gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
-		gconf_client_remove_dir (self->context, GCONF_FONT_TREE, NULL);
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
 		return;
 	}
 
@@ -694,16 +736,26 @@ defaults_init (Defaults* self)
 						     &error);
 	if (error)
 	{
-		gconf_client_notify_remove (self->context, self->notifier[0]);
-		gconf_client_notify_remove (self->context, self->notifier[1]);
-		gconf_client_notify_remove (self->context, self->notifier[2]);
-		gconf_client_notify_remove (self->context, self->notifier[3]);
-		gconf_client_remove_dir (self->context, GCONF_UI_TREE, NULL);
-		gconf_client_remove_dir (self->context, GCONF_FONT_TREE, NULL);
-		g_object_unref (self->context);
-		g_warning ("%s\n", error->message);
+		_handle_error (self, error);
 		return;
 	}
+
+	// hook up notifiier for gravity changes
+	error = NULL;
+	self->notifier[5] = gconf_client_notify_add (self->context,
+						     GCONF_NOSD_TREE,
+						     _gravity_changed,
+						     (gpointer) self,
+						     NULL,
+						     &error);
+	if (error)
+	{
+		_handle_error (self, error);
+		return;
+	}
+
+	// use fixed slot-allocation for async. and sync. bubbles
+	self->slot_allocation = SLOT_ALLOCATION_FIXED;
 }
 
 static void
@@ -815,6 +867,14 @@ defaults_get_property (GObject*    gobject,
 			g_value_set_double (value, defaults->icon_size);
 		break;
 
+		case PROP_GAUGE_SIZE:
+			g_value_set_double (value, defaults->gauge_size);
+		break;
+
+		case PROP_GAUGE_OUTLINE_WIDTH:
+			g_value_set_double (value, defaults->gauge_outline_width);
+		break;
+
 		case PROP_FADE_IN_TIMEOUT:
 			g_value_set_int (value, defaults->fade_in_timeout);
 		break;
@@ -859,7 +919,19 @@ defaults_get_property (GObject*    gobject,
 		break;
 
 		case PROP_PIXELS_PER_EM:
-			g_value_set_int (value, defaults->pixels_per_em);
+			g_value_set_double (value, defaults->pixels_per_em);
+		break;
+
+		case PROP_SYSTEM_FONT_SIZE:
+			g_value_set_double (value, defaults->system_font_size);
+		break;
+
+		case PROP_SCREEN_DPI:
+			g_value_set_double (value, defaults->screen_dpi);
+		break;
+
+		case PROP_GRAVITY:
+			g_value_set_int (value, defaults->gravity);
 		break;
 
 		default :
@@ -1001,6 +1073,14 @@ defaults_set_property (GObject*      gobject,
 			defaults->icon_size = g_value_get_double (value);
 		break;
 
+		case PROP_GAUGE_SIZE:
+			defaults->gauge_size = g_value_get_double (value);
+		break;
+
+		case PROP_GAUGE_OUTLINE_WIDTH:
+			defaults->gauge_outline_width = g_value_get_double (value);
+		break;
+
 		case PROP_FADE_IN_TIMEOUT:
 			defaults->fade_in_timeout = g_value_get_int (value);
 		break;
@@ -1059,7 +1139,19 @@ defaults_set_property (GObject*      gobject,
 		break;
 
 		case PROP_PIXELS_PER_EM:
-			defaults->pixels_per_em = g_value_get_int (value);
+			defaults->pixels_per_em = g_value_get_double (value);
+		break;
+
+		case PROP_SYSTEM_FONT_SIZE:
+			defaults->system_font_size = g_value_get_double (value);
+		break;
+
+		case PROP_SCREEN_DPI:
+			defaults->screen_dpi = g_value_get_double (value);
+		break;
+
+		case PROP_GRAVITY:
+			defaults->gravity = g_value_get_int (value);
 		break;
 
 		default :
@@ -1097,6 +1189,8 @@ defaults_class_init (DefaultsClass* klass)
 	GParamSpec*   property_content_shadow_color;
 	GParamSpec*   property_margin_size;
 	GParamSpec*   property_icon_size;
+	GParamSpec*   property_gauge_size;
+	GParamSpec*   property_gauge_outline_width;
 	GParamSpec*   property_fade_in_timeout;
 	GParamSpec*   property_fade_out_timeout;
 	GParamSpec*   property_on_screen_timeout;
@@ -1108,6 +1202,9 @@ defaults_class_init (DefaultsClass* klass)
 	GParamSpec*   property_text_body_weight;
 	GParamSpec*   property_text_body_size;
 	GParamSpec*   property_pixels_per_em;
+	GParamSpec*   property_system_font_size;
+	GParamSpec*   property_screen_dpi;
+	GParamSpec*   property_gravity;
 
 	gobject_class->constructed  = defaults_constructed;
 	gobject_class->dispose      = defaults_dispose;
@@ -1126,15 +1223,27 @@ defaults_class_init (DefaultsClass* klass)
 		G_TYPE_NONE,
 		0);
 
+	g_defaults_signals[GRAVITY_CHANGED] = g_signal_new (
+		"gravity-changed",
+		G_OBJECT_CLASS_TYPE (gobject_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (DefaultsClass, gravity_changed),
+		NULL,
+		NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE,
+		0);
+
 	property_desktop_width = g_param_spec_int (
 				"desktop-width",
 				"desktop-width",
 				"Width of desktop in pixels",
 				0,
-				4096,
+				G_MAXINT,
 				gdk_screen_get_width (screen),
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_WIDTH,
 					 property_desktop_width);
@@ -1144,10 +1253,11 @@ defaults_class_init (DefaultsClass* klass)
 				"desktop-height",
 				"Height of desktop in pixels",
 				0,
-				4096,
+				G_MAXINT,
 				gdk_screen_get_height (screen),
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_HEIGHT,
 					 property_desktop_height);
@@ -1157,10 +1267,11 @@ defaults_class_init (DefaultsClass* klass)
 				"desktop-top",
 				"Top of desktop in pixels",
 				0,
-				4096,
+				G_MAXINT,
 				0,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_TOP,
 					 property_desktop_top);
@@ -1170,10 +1281,11 @@ defaults_class_init (DefaultsClass* klass)
 				"desktop-bottom",
 				"Bottom of desktop in pixels",
 				0,
-				4096,
-				4096,
+				G_MAXINT,
+				G_MAXINT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_BOTTOM,
 					 property_desktop_bottom);
@@ -1183,10 +1295,11 @@ defaults_class_init (DefaultsClass* klass)
 				"desktop-left",
 				"Left of desktop in pixels",
 				0,
-				4096,
+				G_MAXINT,
 				0,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_LEFT,
 					 property_desktop_left);
@@ -1196,10 +1309,11 @@ defaults_class_init (DefaultsClass* klass)
 				"desktop-right",
 				"Right of desktop in pixels",
 				0,
-				4096,
-				4096,
+				G_MAXINT,
+				G_MAXINT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_RIGHT,
 					 property_desktop_right);
@@ -1212,7 +1326,8 @@ defaults_class_init (DefaultsClass* klass)
 				16.0f,
 				DEFAULT_DESKTOP_BOTTOM_GAP,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_DESKTOP_BOTTOM_GAP,
 					 property_desktop_bottom_gap);
@@ -1225,7 +1340,8 @@ defaults_class_init (DefaultsClass* klass)
 				256.0f,
 				50.0f,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_STACK_HEIGHT,
 					 property_stack_height);
@@ -1238,7 +1354,8 @@ defaults_class_init (DefaultsClass* klass)
 				10.0f,
 				DEFAULT_BUBBLE_VERT_GAP,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_VERT_GAP,
 					 property_bubble_vert_gap);
@@ -1251,7 +1368,8 @@ defaults_class_init (DefaultsClass* klass)
 				10.0f,
 				DEFAULT_BUBBLE_HORZ_GAP,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_HORZ_GAP,
 					 property_bubble_horz_gap);
@@ -1264,7 +1382,8 @@ defaults_class_init (DefaultsClass* klass)
 				256.0f,
 				DEFAULT_BUBBLE_WIDTH,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_WIDTH,
 					 property_bubble_width);
@@ -1277,7 +1396,8 @@ defaults_class_init (DefaultsClass* klass)
 				256.0f,
 				DEFAULT_BUBBLE_MIN_HEIGHT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_MIN_HEIGHT,
 					 property_bubble_min_height);
@@ -1290,7 +1410,8 @@ defaults_class_init (DefaultsClass* klass)
 				256.0f,
 				DEFAULT_BUBBLE_MAX_HEIGHT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_MAX_HEIGHT,
 					 property_bubble_max_height);
@@ -1303,7 +1424,8 @@ defaults_class_init (DefaultsClass* klass)
 				32.0f,
 				DEFAULT_BUBBLE_SHADOW_SIZE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_SHADOW_SIZE,
 					 property_bubble_shadow_size);
@@ -1314,7 +1436,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Color of bubble drop-shadow",
 				DEFAULT_BUBBLE_SHADOW_COLOR,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_SHADOW_COLOR,
 					 property_bubble_shadow_color);
@@ -1325,7 +1448,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Color of bubble-background",
 				DEFAULT_BUBBLE_BG_COLOR,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_BG_COLOR,
 					 property_bubble_bg_color);
@@ -1336,7 +1460,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Opacity of bubble-background",
 				DEFAULT_BUBBLE_BG_OPACITY,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_BG_OPACITY,
 					 property_bubble_bg_opacity);
@@ -1347,7 +1472,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Opacity of bubble in mouse-over case",
 				DEFAULT_BUBBLE_HOVER_OPACITY,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_HOVER_OPACITY,
 					 property_bubble_hover_opacity);
@@ -1360,7 +1486,8 @@ defaults_class_init (DefaultsClass* klass)
 				16.0f,
 				DEFAULT_BUBBLE_CORNER_RADIUS,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_BUBBLE_CORNER_RADIUS,
 					 property_bubble_corner_radius);
@@ -1373,7 +1500,8 @@ defaults_class_init (DefaultsClass* klass)
 				8.0f,
 				DEFAULT_CONTENT_SHADOW_SIZE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_CONTENT_SHADOW_SIZE,
 					 property_content_shadow_size);
@@ -1384,7 +1512,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Color of icon/text drop-shadow",
 				DEFAULT_CONTENT_SHADOW_COLOR,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_CONTENT_SHADOW_COLOR,
 					 property_content_shadow_color);
@@ -1397,7 +1526,8 @@ defaults_class_init (DefaultsClass* klass)
 				32.0f,
 				DEFAULT_MARGIN_SIZE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_MARGIN_SIZE,
 					 property_margin_size);
@@ -1410,11 +1540,40 @@ defaults_class_init (DefaultsClass* klass)
 				64.0f,
 				DEFAULT_ICON_SIZE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_ICON_SIZE,
 					 property_icon_size);
 
+	property_gauge_size = g_param_spec_double (
+				"gauge-size",
+				"gauge-size",
+				"Size/height (in em) of gauge/indicator",
+				0.5f,
+				1.0f,
+				DEFAULT_GAUGE_SIZE,
+				G_PARAM_CONSTRUCT |
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gobject_class,
+					 PROP_GAUGE_SIZE,
+					 property_gauge_size);
+
+	property_gauge_outline_width = g_param_spec_double (
+				"gauge-outline-width",
+				"gauge-outline-width",
+				"Width/thickness (in em) of gauge-outline",
+				0.1f,
+				0.2f,
+				DEFAULT_GAUGE_OUTLINE_WIDTH,
+				G_PARAM_CONSTRUCT |
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gobject_class,
+					 PROP_GAUGE_OUTLINE_WIDTH,
+					 property_gauge_outline_width);
+
 	property_fade_in_timeout = g_param_spec_int (
 				"fade-in-timeout",
 				"fade-in-timeout",
@@ -1423,7 +1582,8 @@ defaults_class_init (DefaultsClass* klass)
 				10000,
 				DEFAULT_FADE_IN_TIMEOUT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_FADE_IN_TIMEOUT,
 					 property_fade_in_timeout);
@@ -1436,7 +1596,8 @@ defaults_class_init (DefaultsClass* klass)
 				10000,
 				DEFAULT_FADE_OUT_TIMEOUT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_FADE_OUT_TIMEOUT,
 					 property_fade_out_timeout);
@@ -1449,7 +1610,8 @@ defaults_class_init (DefaultsClass* klass)
 				10000,
 				DEFAULT_ON_SCREEN_TIMEOUT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_ON_SCREEN_TIMEOUT,
 					 property_on_screen_timeout);
@@ -1460,7 +1622,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Font-face to use of any rendered text",
 				DEFAULT_TEXT_FONT_FACE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_FONT_FACE,
 					 property_text_font_face);
@@ -1471,7 +1634,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Color to use for content title-text",
 				DEFAULT_TEXT_TITLE_COLOR,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_TITLE_COLOR,
 					 property_text_title_color);
@@ -1484,7 +1648,8 @@ defaults_class_init (DefaultsClass* klass)
 				1000,
 				DEFAULT_TEXT_TITLE_WEIGHT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_TITLE_WEIGHT,
 					 property_text_title_weight);
@@ -1497,7 +1662,8 @@ defaults_class_init (DefaultsClass* klass)
 				32.0f,
 				DEFAULT_TEXT_TITLE_SIZE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_TITLE_SIZE,
 					 property_text_title_size);
@@ -1508,7 +1674,8 @@ defaults_class_init (DefaultsClass* klass)
 				"Color to use for content body-text",
 				DEFAULT_TEXT_BODY_COLOR,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_BODY_COLOR,
 					 property_text_body_color);
@@ -1521,7 +1688,8 @@ defaults_class_init (DefaultsClass* klass)
 				1000,
 				DEFAULT_TEXT_BODY_WEIGHT,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_BODY_WEIGHT,
 					 property_text_body_weight);
@@ -1534,23 +1702,67 @@ defaults_class_init (DefaultsClass* klass)
 				32.0f,
 				DEFAULT_TEXT_BODY_SIZE,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_TEXT_BODY_SIZE,
 					 property_text_body_size);
 
-	property_pixels_per_em = g_param_spec_int (
+	property_pixels_per_em = g_param_spec_double (
 				"pixels-per-em",
 				"pixels-per-em",
 				"Number of pixels for one em-unit",
-				1,
-				100,
+				1.0f,
+				100.0f,
 				DEFAULT_PIXELS_PER_EM,
 				G_PARAM_CONSTRUCT |
-				G_PARAM_READWRITE);
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class,
 					 PROP_PIXELS_PER_EM,
 					 property_pixels_per_em);
+
+	property_system_font_size = g_param_spec_double (
+				"system-font-size",
+				"system-font-size",
+				"System font-size in pt",
+				1.0f,
+				100.0f,
+				DEFAULT_SYSTEM_FONT_SIZE,
+				G_PARAM_CONSTRUCT |
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gobject_class,
+					 PROP_SYSTEM_FONT_SIZE,
+					 property_system_font_size);
+
+	property_screen_dpi = g_param_spec_double (
+				"screen-dpi",
+				"screen-dpi",
+				"Screen DPI value",
+				10.0f,
+				600.0f,
+				DEFAULT_SCREEN_DPI,
+				G_PARAM_CONSTRUCT |
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gobject_class,
+					 PROP_SCREEN_DPI,
+					 property_screen_dpi);
+
+	property_gravity = g_param_spec_int (
+				"gravity",
+				"gravity",
+				"Positional hint for placing bubbles",
+				0,
+				2,
+				DEFAULT_GRAVITY,
+				G_PARAM_CONSTRUCT |
+				G_PARAM_READWRITE |
+				G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (gobject_class,
+					 PROP_GRAVITY,
+					 property_gravity);
 }
 
 /*-- public API --------------------------------------------------------------*/
@@ -1563,12 +1775,6 @@ defaults_new (void)
 	return this;
 }
 
-void
-defaults_del (Defaults* self)
-{
-	g_object_unref (self);
-}
-
 gint
 defaults_get_desktop_width (Defaults* self)
 {
@@ -1912,6 +2118,32 @@ defaults_get_icon_size (Defaults* self)
 	return icon_size;
 }
 
+gdouble
+defaults_get_gauge_size (Defaults* self)
+{
+	gdouble gauge_size;
+
+	if (!self || !IS_DEFAULTS (self))
+		return 0.0f;
+
+	g_object_get (self, "gauge-size", &gauge_size, NULL);
+
+	return gauge_size;
+}
+
+gdouble
+defaults_get_gauge_outline_width (Defaults* self)
+{
+	gdouble gauge_outline_width;
+
+	if (!self || !IS_DEFAULTS (self))
+		return 0.0f;
+
+	g_object_get (self, "gauge-outline-width", &gauge_outline_width, NULL);
+
+	return gauge_outline_width;
+}
+
 gint
 defaults_get_fade_in_timeout (Defaults* self)
 {
@@ -2046,37 +2278,70 @@ defaults_get_text_body_size (Defaults* self)
 
 /* we use the normal font-height in pixels ("pixels-per-em") as the measurement
  * for 1 em, thus it should _not_ be called before defaults_constructed() */
-gint
+gdouble
 defaults_get_pixel_per_em (Defaults* self)
 {
 	if (!self || !IS_DEFAULTS (self))
-		return 0;
+		return 0.0f;
 
-	gint pixels_per_em;
+	gdouble pixels_per_em;
 
 	g_object_get (self, "pixels-per-em", &pixels_per_em, NULL);
 
 	return pixels_per_em;
 }
 
+gdouble
+defaults_get_system_font_size (Defaults* self)
+{
+	if (!self || !IS_DEFAULTS (self))
+		return 0.0f;
+
+	gdouble system_font_size;
+
+	g_object_get (self, "system-font-size", &system_font_size, NULL);
+
+	return system_font_size;
+}
+
+gdouble
+defaults_get_screen_dpi (Defaults* self)
+{
+	if (!self || !IS_DEFAULTS (self))
+		return 0.0f;
+
+	gdouble screen_dpi;
+
+	g_object_get (self, "screen-dpi", &screen_dpi, NULL);
+
+	return screen_dpi;
+}
+
 static gboolean
 defaults_multihead_does_focus_follow (Defaults *self)
 {
-	GError *error = NULL;
-	gboolean mode = FALSE;
+	GError*  error = NULL;
+	gboolean mode  = FALSE;
 
 	g_return_val_if_fail (self != NULL && IS_DEFAULTS (self), FALSE);
 
-	gchar *mode_str = gconf_client_get_string (self->context,
+	gchar* mode_str = gconf_client_get_string (self->context,
 						   GCONF_MULTIHEAD_MODE,
 						   &error);
 	if (mode_str != NULL)
 	{
-		if (g_strcmp0 (mode_str, "focus-follow"))
+		if (! g_strcmp0 (mode_str, "focus-follow"))
 			mode = TRUE;
-	} else if (error != NULL)
-		g_warning ("error getting multihead mode: %s\n",
-			   error->message);
+
+		g_free ((gpointer) mode_str);
+	}
+	else if (error != NULL)
+	{
+		g_warning ("defaults_multihead_does_focus_follow(): "
+		           "Got error \"%s\"\n",
+		           error->message);
+		g_error_free (error);
+	}
 	
 	return mode;
 }
@@ -2104,8 +2369,8 @@ _window_look_for_top_panel_attributes (GdkWindow *win)
 		goto failed;
 
 	/* discard dialog windows like panel properties or the applet directory... */
-	if (gdk_window_get_type_hint (win)
-	    != GDK_WINDOW_TYPE_HINT_DOCK)
+	if (wnck_window_get_window_type (wnck_window_get (GDK_WINDOW_XWINDOW (win)))
+	    != WNCK_WINDOW_DOCK)
 		goto failed;
 
 	/* select only the top panel */
@@ -2264,3 +2529,25 @@ defaults_get_top_corner (Defaults *self, gint *x, gint *y)
 
 	g_debug ("top corner at: %d, %d", *x, *y);
 }
+
+Gravity
+defaults_get_gravity (Defaults* self)
+{
+	if (!self || !IS_DEFAULTS (self))
+		return GRAVITY_NONE;
+
+	Gravity gravity;
+
+	g_object_get (self, "gravity", &gravity, NULL);
+
+	return gravity;
+}
+
+SlotAllocation
+defaults_get_slot_allocation (Defaults *self)
+{
+	if (!self || !IS_DEFAULTS (self))
+		return SLOT_ALLOCATION_NONE;
+
+	return self->slot_allocation;
+}
