@@ -10,6 +10,7 @@
 **
 ** Authors:
 **    Cody Russell <cody.russell@canonical.com>
+**    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
 **
 ** This program is free software: you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License version 3, as published
@@ -25,6 +26,34 @@
 **
 *******************************************************************************/
 
-gchar*
-filter_text (const gchar* app_name);
+#include <glib.h>
+#include <cairo.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 
+#define WM_NAME_COMPIZ   "compiz"
+#define WM_NAME_METACITY "Metacity"
+#define WM_NAME_XFCE     "Xfwm4"
+#define WM_NAME_KWIN     "KWin"
+#define WM_NAME_XMONAD   "xmonad"
+
+gchar*
+filter_text (const gchar* text);
+
+gchar*
+newline_to_space (const gchar* text);
+
+cairo_surface_t*
+copy_surface (cairo_surface_t* orig);
+
+gboolean
+destroy_cloned_surface (cairo_surface_t* surface);
+
+gchar*
+get_wm_name (Display* dpy);
+
+guint
+extract_point_size (const gchar* string);
+
+GString*
+extract_font_face (const gchar* string);
